package org.gcube.application.aquamaps.enabling.Impl;

import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.gcube.application.aquamaps.aquamapsservice.stubs.datamodel.environments.ComputationalInfrastructure;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.AquaMapsServiceCall;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.AquaMapsServiceInterface;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.Constant;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.DataManagementCall;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.DataManagementInterface;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.PublisherInterface;
import org.gcube.application.aquamaps.aquamapsservice.stubs.wrapper.PublisherServiceCall;
import org.gcube.application.aquamaps.enabling.Configuration;
import org.gcube.application.aquamaps.enabling.ParameterNotFoundException;
import org.gcube.application.aquamaps.enabling.ScopeNotFoundException;
import org.gcube.application.aquamaps.enabling.Impl.crawler.CrawlerMode;
import org.gcube.application.aquamaps.enabling.Impl.crawler.ISCrawler;
import org.gcube.application.aquamaps.enabling.model.DBDescriptor;
import org.gcube.application.aquamaps.enabling.model.DataSourceDescriptor;
import org.gcube.application.aquamaps.enabling.model.GeoServerDescriptor;
import org.gcube.application.aquamaps.enabling.model.VODescriptor;
import org.gcube.common.core.scope.GCUBEScope;
import org.gcube.common.core.utils.logging.GCUBELog;

public class ConfigurationImpl implements Configuration {

	
	
	protected static GCUBELog logger= new GCUBELog(ConfigurationImpl.class);
	
	private static final int DEFAULT_REFRESH_INTERVAL=10;
	
	private static final String FETCH_INTERVAL_MINUTES="FETCH_INTERVAL_MINUTES";
	

	private ISCrawler crawler;
	
	
	private static ConfigurationImpl impl=null;
	
	
	public static synchronized Configuration getSingle(GCUBEScope defaultScope)throws Exception{
		if (impl==null) impl=new ConfigurationImpl(CrawlerMode.SINGLESCOPE,defaultScope);
		else logger.debug("Instance was already created, crawler is "+impl.crawler.getClass().getName());
		return impl;
	}
	
	public static synchronized Configuration get(CrawlerMode mode)throws Exception{
		if(impl==null) impl= new ConfigurationImpl(mode, null);
		else logger.debug("Instance was already created, crawler is "+impl.crawler.getClass().getName());
		return impl;
	}
	
	
	private ConfigurationImpl(CrawlerMode mode, GCUBEScope defaultScope) throws Exception{
		Integer interval=DEFAULT_REFRESH_INTERVAL;
		try{
			Properties prop=new Properties();
			prop.load(Configuration.class.getResourceAsStream("util/resources/config.properties"));
			interval=Integer.parseInt(prop.getProperty(FETCH_INTERVAL_MINUTES));
		}catch(Exception e){
			logger.warn("Unable to read properties file under resources package. Gonna use default ("+interval+")",e);
		}
		crawler=ISCrawler.get(interval,mode,defaultScope);
	}
	
	
	
	
	protected VODescriptor getVO(GCUBEScope scope) throws ScopeNotFoundException{
		VODescriptor toReturn=null;
		if(crawler.getInfrastructureMap().containsKey(scope)) toReturn=crawler.getInfrastructureMap().get(scope);
		if(toReturn==null) throw new ScopeNotFoundException();
		else return toReturn;
	}

	@Override
	public DataManagementInterface getDMInterface(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException,Exception {
		try{
			return DataManagementCall.getCall(scope, getVO(scope).getRunningInstance().getAccessPoint().getEndpoint(Constant.DATAMANAGEMENT_PT_NAME).getAddress().toString(), false);
		}catch(ScopeNotFoundException e){throw e;}
		catch(Exception e){throw new ParameterNotFoundException();}
	}

	@Override
	public AquaMapsServiceInterface getAMInterface(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException,Exception {
		try{
			return AquaMapsServiceCall.getCall(scope, getVO(scope).getRunningInstance().getAccessPoint().getEndpoint(Constant.AQUAMAPSSERVICE_PT_NAME).getAddress().toString(), false);
		}catch(ScopeNotFoundException e){throw e;}
		catch(Exception e){throw new ParameterNotFoundException();}
	}

	@Override
	public List<GeoServerDescriptor> getGeoServers(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException {
		List<GeoServerDescriptor> toReturn=getVO(scope).getGeoServers();
		if(toReturn==null||toReturn.size()==0) throw new ParameterNotFoundException();
		else return toReturn;
	}

	@Override
	public String getServiceConfigurationParameter(GCUBEScope scope,
			String parameter) throws ParameterNotFoundException,
			ScopeNotFoundException {
		String toReturn=getVO(scope).getServiceParam(parameter);
		if(toReturn==null) throw new ParameterNotFoundException();
		else return toReturn;
	}

	

	@Override
	public List<ComputationalInfrastructure> getAvailableInfrastructures(
			GCUBEScope scope) throws ParameterNotFoundException,
			ScopeNotFoundException {
		List<ComputationalInfrastructure> toReturn=getVO(scope).getInfrastructures();
		if(toReturn==null||toReturn.size()==0) throw new ParameterNotFoundException();
		else return toReturn;
	}

	@Override
	public Set<GCUBEScope> getAvailableScopes() throws Exception {
		return crawler.getInfrastructureMap().keySet();
	}
	
	@Override
	public String getBackendUrl(GCUBEScope scope, String backend)
			throws ParameterNotFoundException, ScopeNotFoundException {
		String toReturn= getVO(scope).getBackendsUrl().get(backend);
		if(toReturn==null)throw new ParameterNotFoundException();
		return toReturn;
	}
	@Override
	public DBDescriptor getGeoServerDb(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException {
		DBDescriptor toReturn= getVO(scope).getGeoDb();
		if(toReturn==null)throw new ParameterNotFoundException();
		return toReturn;
	}
	@Override
	public DBDescriptor getInternalDB(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException {
		DBDescriptor descriptor=getVO(scope).getInternalDB();
		if(descriptor==null) throw new ParameterNotFoundException();
		return descriptor;
	}
	@Override
	public DataSourceDescriptor getGeoNetwork(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException {
		DataSourceDescriptor descriptor= getVO(scope).getGeoNetwork();
		if(descriptor==null) throw new ParameterNotFoundException();
		return descriptor;
	}
	@Override
	public GeoServerDescriptor getGeoServerByEntryPoint(GCUBEScope scope,String entryPoint)
			throws ParameterNotFoundException, ScopeNotFoundException {
		GeoServerDescriptor toReturn= getVO(scope).getGeoServerByEntryPoint(entryPoint);
		if(toReturn==null)throw new ParameterNotFoundException();
		return toReturn;
	}
	@Override
	public DBDescriptor getPublisherDataBase(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException {
		DBDescriptor descriptor=getVO(scope).getPublisherDB();
		if(descriptor==null) throw new ParameterNotFoundException();
		return descriptor;
	}
	@Override
	public PublisherInterface getPublisherInterface(GCUBEScope scope)
			throws ParameterNotFoundException, ScopeNotFoundException,
			Exception {
		try{
			return PublisherServiceCall.getCall(scope, getVO(scope).getRunningInstance().getAccessPoint().getEndpoint(Constant.PUBLISHERSERVICE_PT_NAME).getAddress().toString(), false);
		}catch(ScopeNotFoundException e){throw e;}
		catch(Exception e){throw new ParameterNotFoundException();}
	}
	

}
